// Copyright (c) 2020, QuantStack and XVega Contributors
//
// Distributed under the terms of the BSD 3-Clause License.
//
// The full license is in the file LICENSE, distributed with this software.

#ifndef XVEGA_CONFIG_CLING_HPP
#define XVEGA_CONFIG_CLING_HPP

#pragma cling add_library_path("D:/bld/xvega_1611327629804/_h_env/Library/lib")
#pragma cling load("libxvega")

#endif
